@ECHO OFF
ECHO OFF
GOTO Begin
REM
REM     Program          :  FileUtil.bat
REM
REM     Version          :  1.16
REM     Started on       :  November 24, 2007
REM     Completed on     :  November 24, 2007
REM
REM     Description      :  Library of file utilities
REM
REM     Required modules :  Date2.exe, Wc.exe, Str2Env.exe, StrComp.exe,
REM                         Tail.exe, StrUtil.bat
REM     Dos required     :  Dos 6.0 or later
REM
REM     Notes - usage    :  See help (/? parameter)
REM                         Setting FU_DEBUG environment variable to "Y", enable
REM                         on-screen commands echoing
REM                         Setting DEBUG environment variable to "Y", enable
REM                         on-screen commands echoing on return to caller
REM     References       :  None
REM
REM     Authors          :  J.Hubb - planning
REM                         J.Hubb - programming
REM                         J.Hubb - debug session
REM     Authors address  :  J.Hubb
REM                         Indigo skyline
REM                         A'DAAR channel
REM

:Begin
IF "%FU_DEBUG%" == "Y" ECHO ON
SET FU_VER=1.16

:ChkParams
IF "%1" == "" GOTO Help
IF "%1" == "/?" GOTO Help
FOR %%N IN (NewFileName ReadTextRow Ver) DO IF "%1" == "%%N" GOTO %1
GOTO FuncErr

REM
REM Function :  NewFileName
REM Desc.    :  Creates a new file with a non-existent name in the
REM             directory and the extension specified; the name has
REM             the format "pmmssx", where p is the specified prefix,
REM             x is a incremental letter ("a" .. "z"), mm:ss is the
REM             current timestamp; note that no other file with name
REM             "pmmssx*.%4" exists in the directory %2; the new name
REM             is returned in the specified env. variable
REM
REM Input    :  %2  directory of the new file
REM             %3  prefix of one or more chars for the name of the new file
REM             %4  extension of the new file
REM             %5  var. name to be set
REM Output   :  environment variable named "%5"
REM
:NewFileName
    IF "%5" == "" GOTO ParamErr
    SET FU_NFN=%3mmss
    IF NOT "%OS%" == "" GOTO NFNWinNt
:NFNWin9x
    DATE2 "+%3%%M%%S" | STR2ENV "" FU_NFN > NUL
    GOTO NFNChkName
:NFNWinNt
    FOR /F "usebackq" %%N IN (`DATE2 "+%3%%M%%S"`) DO SET FU_NFN=%%N
:NFNChkName
    FOR %%N IN (a b c d e f g h i j k l m n o p q r s t u v w x y z) DO IF NOT EXIST %2\%FU_NFN%%%N*.%4 GOTO NFN%%N
    GOTO NewFileName
:NFNa
    SET FU_NFN=%FU_NFN%a
    GOTO NFNLastChk
:NFNb
    SET FU_NFN=%FU_NFN%b
    GOTO NFNLastChk
:NFNc
    SET FU_NFN=%FU_NFN%c
    GOTO NFNLastChk
:NFNd
    SET FU_NFN=%FU_NFN%d
    GOTO NFNLastChk
:NFNe
    SET FU_NFN=%FU_NFN%e
    GOTO NFNLastChk
:NFNf
    SET FU_NFN=%FU_NFN%f
    GOTO NFNLastChk
:NFNg
    SET FU_NFN=%FU_NFN%g
    GOTO NFNLastChk
:NFNh
    SET FU_NFN=%FU_NFN%h
    GOTO NFNLastChk
:NFNi
    SET FU_NFN=%FU_NFN%i
    GOTO NFNLastChk
:NFNj
    SET FU_NFN=%FU_NFN%j
    GOTO NFNLastChk
:NFNk
    SET FU_NFN=%FU_NFN%k
    GOTO NFNLastChk
:NFNl
    SET FU_NFN=%FU_NFN%l
    GOTO NFNLastChk
:NFNm
    SET FU_NFN=%FU_NFN%m
    GOTO NFNLastChk
:NFNn
    SET FU_NFN=%FU_NFN%n
    GOTO NFNLastChk
:NFNo
    SET FU_NFN=%FU_NFN%o
    GOTO NFNLastChk
:NFNp
    SET FU_NFN=%FU_NFN%p
    GOTO NFNLastChk
:NFNq
    SET FU_NFN=%FU_NFN%q
    GOTO NFNLastChk
:NFNr
    SET FU_NFN=%FU_NFN%r
    GOTO NFNLastChk
:NFNs
    SET FU_NFN=%FU_NFN%s
    GOTO NFNLastChk
:NFNt
    SET FU_NFN=%FU_NFN%t
    GOTO NFNLastChk
:NFNu
    SET FU_NFN=%FU_NFN%u
    GOTO NFNLastChk
:NFNv
    SET FU_NFN=%FU_NFN%v
    GOTO NFNLastChk
:NFNw
    SET FU_NFN=%FU_NFN%w
    GOTO NFNLastChk
:NFNx
    SET FU_NFN=%FU_NFN%x
    GOTO NFNLastChk
:NFNy
    SET FU_NFN=%FU_NFN%y
    GOTO NFNLastChk
:NFNz
    SET FU_NFN=%FU_NFN%z
    GOTO NFNLastChk
:NFNLastChk
    IF EXIST %2\%FU_NFN%*.%4 GOTO NewFileName
    ECHO FileUtil.bat >> %2\%FU_NFN%.%4
    ECHO NewFileName: >> %2\%FU_NFN%.%4
    ECHO temporary file >> %2\%FU_NFN%.%4
    ECHO collision test >> %2\%FU_NFN%.%4
    WC -l < %2\%FU_NFN%.%4 | STRCOMP "=" "      4" > NUL
    IF ERRORLEVEL 1 GOTO NewFileName
    SET %5=%FU_NFN%> %2\%FU_NFN%.%4
:NFNEnd
    SET FU_NFN=
    GOTO EndSub

REM
REM Function :  ReadTextRow
REM Desc.    :  Opens a text file (Mode = "Open") and reads one row for
REM             each call (Mode = "Next"), storing it in the env. var.
REM             specified in %4; before exiting, the text file should
REM             be closed (Mode = "Close"); the status env. variable
REM             specified in %4 (or %3) contains "EOF" when all the file
REM             is being read, otherwise contains temporary data
REM
REM Input    :  %2  operation mode; can be:
REM                     Open    sets status env. variable and creates
REM                             temporary files (all .bat)
REM                     Next    returns the next text row; if EOF is.
REM                             reached, sets status env. var. to "EOF"
REM                     Close   sets status env. var. to "EOF" and remove
REM                             temporary files
REM             %3  according to Mode parameter (%2) value, must be:
REM                     Open    pathname of text file to be read
REM                     Next    status of the call with %2 == Open
REM                     Close   status of the call with %2 == Open
REM             %4  according to Mode parameter (%2) value, must be:
REM                     Open    env. var. name to contain open status;
REM                     Next    env. var. name to contain the text of
REM                             next row; it must be filled using as many
REM                             characters as max length
REM Output   :  environment variable named "%4" (open status if %2 == Open
REM             else next row)
REM
:ReadTextRow
    GOTO RTR%2
:RTROpen
    IF "%4" == "" GOTO ParamErr
    CALL FileUtil NewFileName %TMP% FU bat FU_RTR
    ECHO.> %TMP%\%FU_RTR%1.bat
    ECHO.> %TMP%\%FU_RTR%2.bat
    ECHO @ECHO OFF> %TMP%\%FU_RTR%.bat
    ECHO IF "%%FU_DEBUG%%" == "Y" ECHO ON>> %TMP%\%FU_RTR%.bat
    ECHO.>> %TMP%\%FU_RTR%.bat
    ECHO :ChkEof>> %TMP%\%FU_RTR%.bat
    ECHO.| STRINS 1 "WC -l $3C %TMP%\%FU_RTR%1.bat $7C STRCOMP $22$3C$3E$22 $22      0$22 $3E NUL">> %TMP%\%FU_RTR%.bat
    ECHO IF ERRORLEVEL 1 GOTO Eof>> %TMP%\%FU_RTR%.bat
    ECHO.>> %TMP%\%FU_RTR%.bat
    ECHO :GetLn>> %TMP%\%FU_RTR%.bat
    ECHO CALL StrUtil Str2Env %TMP%\%FU_RTR%1.bat %%1>> %TMP%\%FU_RTR%.bat
    ECHO.| STRINS 1 "TAIL +2 $3C %TMP%\%FU_RTR%1.bat $3E %TMP%\%FU_RTR%2.bat">> %TMP%\%FU_RTR%.bat
    ECHO.| STRINS 1 "MOVE /Y %TMP%\%FU_RTR%2.bat %TMP%\%FU_RTR%1.bat $3E NUL">> %TMP%\%FU_RTR%.bat
    ECHO.| STRINS 1 "ECHO.$3E %TMP%\%FU_RTR%2.bat">> %TMP%\%FU_RTR%.bat
    ECHO GOTO End>> %TMP%\%FU_RTR%.bat
    ECHO.>> %TMP%\%FU_RTR%.bat
    ECHO :Eof>> %TMP%\%FU_RTR%.bat
    ECHO SET %4=EOF>> %TMP%\%FU_RTR%.bat
    ECHO SET %%2=EOF>> %TMP%\%FU_RTR%.bat
    ECHO.>> %TMP%\%FU_RTR%.bat
    ECHO :End>> %TMP%\%FU_RTR%.bat
    TYPE %3 > %TMP%\%FU_RTR%1.bat
    SET %4=%FU_RTR%
    GOTO RTREnd
:RTRNext
    IF "%4" == "" GOTO ParamErr
    IF "%3" == "EOF" GOTO RTREnd
    CALL %TMP%\%3.bat %4 FU_RTR
    IF "%FU_RTR%" == "EOF" GOTO RTRClose
    GOTO RTREnd
:RTRClose
    IF "%3" == "" GOTO ParamErr
    IF "%3" == "EOF" GOTO RTREnd
    REM Set to 0 bytes the file %31.bat
    DEL %TMP%\%32.bat > %TMP%\%31.bat
    REM Set status env. var. to "EOF"
    CALL %TMP%\%3.bat FU_RTR FU_RTR
    DEL %TMP%\%31.bat
    DEL %TMP%\%3.bat
    GOTO RTREnd
:RTREnd
    SET FU_RTR=
    GOTO EndSub

REM
REM Function :  Ver
REM Desc.    :  Sets the specified variable to FileUtil version number
REM
REM Input    :  %2  var. name to be set
REM Output   :  environment variable named "%2"
REM
:Ver
    IF "%2" == "" GOTO ParamErr
    SET %2=%FU_VER%
    GOTO EndSub

:FuncErr
ECHO.
ECHO FileUtil.bat %FU_VER%.
ECHO.
ECHO Error: function "%1" does not exist.
ECHO Execution aborted.
PAUSE > NUL
GOTO End

:ParamErr
ECHO.
ECHO FileUtil.bat %FU_VER%.
ECHO.
ECHO Error: function "%1" called with wrong parameter number (see Help).
ECHO Execution aborted.
PAUSE > NUL
GOTO End

:Help
ECHO.
ECHO FileUtil.bat %FU_VER%; Open Source library on November 24, 2007 by J.Hubb.
ECHO.
ECHO Syntax:
ECHO   FILEUTIL {FunctionName} [FuncParam0 [FuncParam1 [...]]]
ECHO.
ECHO Calls the specified function {FunctionName} with optional
ECHO or requested parameters [FuncParam0], etc...
ECHO Defined functions are:
ECHO   NewFileName {Dir} {NamePrefix} {Ext} {EnvVarName}
ECHO   ReadTextRow Open {TextFileName} {StatusEnvVarName}
ECHO               Next {StatusEnvVar} {RowEnvVarName}
ECHO               Close {StatusEnvVar}
ECHO   Ver         {EnvVarName}
ECHO.
GOTO End

:End
SET FU_VER=

:EndSub
IF "%DEBUG%" == "Y" ECHO ON
